/*  Peakholder Example - Using USB MIDI with HIDUINO
 *  http://www.pjrc.com/peakholder/
 *
 *  You must first download this sketch to your Arduino Uno,
 *  then reprogram the 8u2 chip with HIDUINO.  Please refer
 *  to the HIDUINO website for details:
 *  http://mtiid.calarts.edu/research/hiduino
 *
 *  This example code is in the public domain.
 */

// this example is untested - TODO: remove this line after testing

#include <Peakholder.h>
#include <HIDUINO_MIDI_Lib.h>

/*******************************************************/
/*  Configuration Variables                            */
/*******************************************************/

// the number of analog inputs connected
const int numberOfInputs = 4;

// which MIDI note number to send for each input
const byte noteNumber[numberOfInputs] =
	{128, 120, 122, 110};

// how long the note will play (time in ms before "note off" is sent)
const int noteDuration[numberOfInputs] = 
	{200, 100, 120, 150};


/*******************************************************/
/*  Normal Variables                                   */
/*******************************************************/

// the millis() time when each "note on" message was transmitted
unsigned long noteOnTime[numberOfInputs];

// the last velocity sent, or zero if the note is off
unsigned char noteVelocity[numberOfInputs];


/*******************************************************/
/*  Program                                            */
/*******************************************************/

void setup() {
  Serial.begin(9600);
  Peakholder.begin(numberOfInputs);
}

void loop() {
  byte inputNumber;
  int value;
  byte velocity;

  if (Peakholder.available()) {
    inputNumber = Peakholder.getInputNumber();
    value = Peakholder.readValue();
    velocity = Peakholder.analog2velocity(value, 36);
    turn_note_on(inputNumber, velocity);
  }
  turn_notes_off();
}

// Send a MIDI "note on" message
//
void turn_note_on(byte inputNumber, byte velocity) {
  // only send a note on message when the note is off
  // or currently on at a lower intensity
  if (noteVelocity[inputNumber] < velocity) {
    // If the note was already on, turn it off before
    // turning it on to the new intensity.  This ensures
    // every note on message has a corresponding note off.
    if (noteVelocity[inputNumber] > 0) {
      MIDI.sendNoteOff(noteNumber[inputNumber], 0);
    }
    noteVelocity[inputNumber] = velocity;
    noteOnTime[inputNumber] = millis();
    MIDI.sendNoteOn(noteNumber[inputNumber], velocity);
  }
}

// Send MIDI "note off" messages, when it's time to do so
//
void turn_notes_off() {
  byte i;
  unsigned long tnow;

  tnow = millis();
  for (i=0; i < numberOfInputs; i++) {
    // if the a note on message is playing
    if (noteVelocity[i] > 0) {
      // this input's note is on...
      // is it time to turn it off?
      if (tnow - noteOnTime[i] > noteDuration[i]) {
        MIDI.sendNoteOff(noteNumber[i], 0);
        noteVelocity[i] = 0;
      }
    }
  }
}

