

#define MAX_EVENTS 100
unsigned long  event_do_at[MAX_EVENTS];
byte           event_target[MAX_EVENTS];



// Turn on an output for a specified duration
//
void output_on(byte out, unsigned long duration) {
  if (out < 1 || out > 6) return;
  unsigned long now = millis();
  unsigned long off_at = now + duration;
  if (events_available(1) && off_at > now) {
    digitalWrite(out - 1, HIGH);
    event_schedule(out + 6, off_at);
  }
}

// Wait for all activity on an output to play out
//
void output_wait(byte out) {
  
  
}

// Turn on an output for a specified duration,
// but not right now, do it after a delay
//
void output_on(byte out, unsigned long duration, unsigned long dly) {
  if (out < 1 || out > 6) return;
  unsigned long now = millis();
  unsigned long on_at = now + dly;
  unsigned long off_at = on_at + duration;
  if (events_available(2) && on_at > now && off_at > now) {
    event_schedule(out, on_at);
    event_schedule(out + 6, off_at);
  }
}

// These scheduled on/off times don't just happen
// by themselves!  This update function must be
// called regularly to actually perform the work.
//
void events_update(void)
{
  unsigned long now = millis();
  for (byte i=0; i<MAX_EVENTS; i++) {
    byte out =  event_target[i];
    // 0 is no event scheduled
    if (out == 0 || now < event_do_at[i]) {
       continue;
    }
    // 1 to 6 are "output on" events
    else if (out >= 1 && out <= 6) {
      digitalWrite(out - 1, HIGH);
      event_target[i] = 0;
    // 7 to 12 are "output off" events
    } else if (out >= 7 && out <= 12) {
      digitalWrite(out - 7, LOW);
      event_target[i] = 0;
    // others should never occur....
    } else {
      Serial.println("Error: unknown event target: ");
      Serial.println(event_target[i]);
      event_target[i] = 0;
    }
  }
}

void outputs_off(void) {
  pinMode(0, OUTPUT);
  digitalWrite(0, LOW);
  pinMode(1, OUTPUT);
  digitalWrite(1, LOW);
  pinMode(2, OUTPUT);
  digitalWrite(2, LOW);
  pinMode(3, OUTPUT);
  digitalWrite(3, LOW);
  pinMode(4, OUTPUT);
  digitalWrite(4, LOW);
  pinMode(5, OUTPUT);
  digitalWrite(5, LOW);
  for (int i=0; i<MAX_EVENTS; i++) {
    event_target[i] = 0;
  }
}



boolean events_available(byte num)
{
  for (byte i=0; i<MAX_EVENTS; i++) {
    if (event_target[i] == 0) {
      if (--num == 0) return true; 
    }
  }
  return false;
}

void event_schedule(byte target, unsigned long at) {
   for (byte i=0; i<MAX_EVENTS; i++) {
     if (event_target[i] == 0) {
       //Serial.println(String("Schedule: ") + (int)target);
       event_target[i] = target;
       event_do_at[i] = at;
       return;
     }
   }
}

